@@ -24,7 +24,7 @@ use Audio::LADSPA::Buffer;
 use 5.006;
 use Carp;
 
-our $VERSION = sprintf("%d.%03d", '$Name: v0_016-2006-05-22 $' =~ /(\d+)_(\d+)/,0,0);
+our $VERSION = sprintf("%d.%03d", '$Name: v0_018-2006-06-15b $' =~ /(\d+)_(\d+)/,0,0);
 
 our @LIBRARIES;	    # will store the list of found libraries as Perl class names
 our @PLUGINS;	    # will store the names of all loaded plugins as Perl class names
@@ -38,13 +38,15 @@ unless (@LIBRARIES) {
 
 sub load {
     my ($class,$lib_path) = @_;
-    my $lib = Audio::LADSPA::LibraryLoader->load($lib_path);
-    push @LIBRARIES,$lib;
-    for ($lib->plugins()) {
-	push @PLUGINS,$_;
-	$PLUGINS{ $_->id } = $_;
+    if (my $lib = Audio::LADSPA::LibraryLoader->load($lib_path)) {
+        push @LIBRARIES,$lib;
+        for ($lib->plugins()) {
+            push @PLUGINS,$_;
+            $PLUGINS{ $_->id } = $_;
+        }
+        return $lib;
     }
-    return $lib;
+    return;
 }
 
 
